import { useState, useEffect, useRef, useCallback, lazy, Suspense } from "react";
import { Textarea } from "@/components/ui/textarea";
import { Button } from "@/components/ui/button";
import { BurnButton } from "@/components/BurnButton";
import { AnimationStyleSelector } from "@/components/AnimationStyleSelector";
import { Timer } from "@/components/Timer";
import { useJournal } from "@/lib/useJournal";
// Temporarily removed useToast to fix React hooks error
import { getRandomQuote, type Quote } from "@/lib/quotes";
import { getDefaultAnimationStyle, AnimationStyle } from "@/lib/animationStyles";
import { Settings, Maximize2, Minimize2, Type, Sun, Moon } from "lucide-react";

// Lazy load heavy components to reduce LCP and unused JavaScript
const FireAnimation = lazy(() => import("@/components/FireAnimation").then(m => ({ default: m.FireAnimation })));
const MotivationalQuote = lazy(() => import("@/components/MotivationalQuote").then(m => ({ default: m.MotivationalQuote })));

// Define a matching constant to the one in useJournal.ts
const EXPIRATION_TIME = 2 * 60 * 1000; // 2 minutes in milliseconds

export function JournalEditor() {
  const textareaRef = useRef<HTMLTextAreaElement>(null);
  const [charCount, setCharCount] = useState(0);
  const [showQuote, setShowQuote] = useState(false);
  const [currentQuote, setCurrentQuote] = useState<Quote | null>(null);
  const [journalBeforeBurn, setJournalBeforeBurn] = useState(""); // Store content before burning
  const [showStyleSelector, setShowStyleSelector] = useState(false);
  const [animationStyle, setAnimationStyle] = useState<AnimationStyle>(getDefaultAnimationStyle());
  const styleSelectorRef = useRef<HTMLDivElement>(null);
  
  // This controls if the animation component is actually in the DOM
  const [isAnimating, setIsAnimating] = useState(false);
  
  // Keep track of animation key to force re-mount when needed
  const [animationKey, setAnimationKey] = useState(0);
  
  // Toast functionality temporarily disabled to fix React hooks error
  
  const [shouldShowTimer, setShouldShowTimer] = useState(false);
  const [displayTimeRemaining, setDisplayTimeRemaining] = useState(0);
  
  // Distraction-free mode and font size settings
  const [isDistractionFree, setIsDistractionFree] = useState(false);
  const [fontSize, setFontSize] = useState<'small' | 'normal' | 'large' | 'huge'>('normal');
  const [showFontOptions, setShowFontOptions] = useState(false);
  const [isLightMode, setIsLightMode] = useState(false);
  const fontOptionsRef = useRef<HTMLDivElement>(null);
  
  const {
    journalContent,
    isBurning,
    timeRemaining,
    isCountingDown,
    handleTextareaInput,
    handleBurn: originalHandleBurn,
    handleBurnComplete: originalHandleBurnComplete
  } = useJournal({
    onExpire: () => {
      // Journal expired - temporarily removed toast notification
      
      // Save the journal content for the fire animation
      if (journalContent.trim().length > 0) {
        setJournalBeforeBurn(journalContent);
        // Trigger the animation
        setIsAnimating(true);
        // Increment animation key
        setAnimationKey(prev => prev + 1);
      }
    }
  });

  // Clear the journal content
  const clearContent = useCallback(() => {
    handleTextareaInput("");
  }, [handleTextareaInput]);

  // Handle burn button click
  const handleBurn = useCallback(() => {
    if (journalContent.trim().length === 0) return;
    
    // Save the current journal content for animation
    setJournalBeforeBurn(journalContent);
    
    // Clear the content immediately
    clearContent();
    
    // Trigger the animation
    setIsAnimating(true);
    
    // Set the burning state in journal hook
    originalHandleBurn();
    
    // Increment animation key to force a fresh animation if user clicks burn again
    setAnimationKey(prev => prev + 1);
  }, [journalContent, clearContent, originalHandleBurn]);

  // Update character count when content changes
  useEffect(() => {
    setCharCount(journalContent.length);
  }, [journalContent]);

  // Handle burn completion and show motivational quote
  const handleBurnComplete = useCallback(() => {
    // Animation is complete, remove it from DOM
    setIsAnimating(false);
    
    // Get a random quote to display
    const quote = getRandomQuote();
    setCurrentQuote(quote);
    
    // Reset the burning state
    originalHandleBurnComplete();
    
    // Show the motivational quote
    setShowQuote(true);
  }, [originalHandleBurnComplete]);

  // Close the quote modal
  const closeQuote = useCallback(() => {
    setShowQuote(false);
    setCurrentQuote(null);
  }, []);

  // Handle animation style change
  const handleStyleChange = useCallback((style: AnimationStyle) => {
    setAnimationStyle(style);
    setShowStyleSelector(false);
    
    // Animation style updated - toast notification temporarily removed
  }, []);

  // Font size utility functions
  const getFontSizeClass = (size: string) => {
    switch (size) {
      case 'small': return 'text-sm';
      case 'normal': return 'text-base';
      case 'large': return 'text-lg';
      case 'huge': return 'text-xl';
      default: return 'text-base';
    }
  };

  const getFontSizeLabel = (size: string) => {
    switch (size) {
      case 'small': return 'Small';
      case 'normal': return 'Normal';
      case 'large': return 'Large';
      case 'huge': return 'Huge';
      default: return 'Normal';
    }
  };

  // Handle font size change
  const handleFontSizeChange = useCallback((newSize: 'small' | 'normal' | 'large' | 'huge') => {
    setFontSize(newSize);
    setShowFontOptions(false);
  }, []);

  // Toggle distraction-free mode
  const toggleDistractionFree = useCallback(() => {
    setIsDistractionFree(prev => !prev);
  }, []);

  // Toggle light/dark mode
  const toggleLightMode = useCallback(() => {
    setIsLightMode(prev => !prev);
  }, []);
  
  // Effect to update our local timer display state
  useEffect(() => {
    // Always show timer if the hook says it should be shown
    setShouldShowTimer(isCountingDown);
    setDisplayTimeRemaining(timeRemaining);
    
    // Force display if timeRemaining is less than the full time (meaning countdown has started)
    if (timeRemaining < EXPIRATION_TIME) {
      setShouldShowTimer(true);
    }
  }, [isCountingDown, timeRemaining]);
  
  // Handle outside clicks for dropdowns
  useEffect(() => {
    function handleClickOutside(event: MouseEvent) {
      if (styleSelectorRef.current && 
          !styleSelectorRef.current.contains(event.target as Node) &&
          showStyleSelector) {
        setShowStyleSelector(false);
      }
      
      if (fontOptionsRef.current && 
          !fontOptionsRef.current.contains(event.target as Node) &&
          showFontOptions) {
        setShowFontOptions(false);
      }
    }
    
    // Add when any selector is shown
    if (showStyleSelector || showFontOptions) {
      document.addEventListener('mousedown', handleClickOutside);
    }
    
    // Clean up
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [showStyleSelector, showFontOptions]);

  // Theme-based styling
  const getThemeClasses = () => {
    if (isLightMode) {
      return {
        container: isDistractionFree 
          ? "fixed inset-0 z-50 bg-white p-8 flex flex-col"
          : "relative my-4",
        textarea: `focus:outline-none resize-none border-none bg-gray-50 text-gray-900 placeholder:text-gray-500`,
        controls: "text-gray-600 hover:text-gray-900 hover:bg-gray-100",
        dropdown: "bg-white border-gray-300 text-gray-900",
        dropdownItem: "hover:bg-gray-100",
        gradient: "bg-gradient-to-r from-orange-500 to-red-600"
      };
    } else {
      return {
        container: isDistractionFree 
          ? "fixed inset-0 z-50 bg-[#121212] p-8 flex flex-col"
          : "relative my-4",
        textarea: `focus:outline-none resize-none border-none bg-[rgba(255,255,255,0.12)] text-[#F5F5F5] placeholder:text-[rgba(245,245,245,0.5)]`,
        controls: "text-[#BDBDBD] hover:text-[#F5F5F5] hover:bg-[rgba(255,255,255,0.1)]",
        dropdown: "bg-[#2d2d2d] border-gray-600 text-[#E0E0E0]",
        dropdownItem: "hover:bg-[rgba(255,255,255,0.1)]",
        gradient: "bg-gradient-to-r from-[#FF5722] to-[#FFC107]"
      };
    }
  };

  const theme = getThemeClasses();
  
  const textareaClass = isDistractionFree
    ? `w-full p-8 rounded-lg ${theme.textarea} ${getFontSizeClass(fontSize)} resize-none`
    : `w-full h-64 sm:h-80 md:h-96 p-4 sm:p-6 rounded-lg ${theme.textarea} ${getFontSizeClass(fontSize)} resize-none`;

  return (
    <>
      <div className={theme.container}>
        {/* Controls Bar */}
        <div className={`flex items-center justify-between mb-4 ${isDistractionFree ? 'mb-6' : ''}`}>
          <div className="flex items-center space-x-2">
            {/* Font Size Control */}
            <div className="relative" ref={fontOptionsRef}>
              <Button
                variant="ghost"
                size="sm"
                onClick={() => setShowFontOptions(!showFontOptions)}
                className={theme.controls}
              >
                <Type className="w-4 h-4 mr-2" />
                {getFontSizeLabel(fontSize)}
              </Button>
              
              {showFontOptions && (
                <div className={`absolute top-full left-0 mt-2 ${theme.dropdown} border rounded-lg shadow-lg z-10 min-w-[120px]`}>
                  {(['small', 'normal', 'large', 'huge'] as const).map((size) => (
                    <button
                      key={size}
                      onClick={() => handleFontSizeChange(size)}
                      className={`w-full px-4 py-2 text-left ${theme.dropdownItem} transition-colors ${
                        fontSize === size ? 'text-orange-400' : ''
                      } ${getFontSizeClass(size)}`}
                    >
                      {getFontSizeLabel(size)}
                    </button>
                  ))}
                </div>
              )}
            </div>

            {/* Animation Style Control */}
            <div className="relative" ref={styleSelectorRef}>
              <Button
                variant="ghost"
                size="sm"
                onClick={() => setShowStyleSelector(!showStyleSelector)}
                className={theme.controls}
              >
                <Settings className="w-4 h-4 mr-2" />
                {animationStyle.name}
              </Button>
              
              {showStyleSelector && (
                <div className="absolute top-full left-0 mt-2 z-10">
                  <AnimationStyleSelector 
                    onSelectStyle={handleStyleChange}
                    currentStyle={animationStyle}
                  />
                </div>
              )}
            </div>

            {/* Light/Dark Mode Toggle */}
            <Button
              variant="ghost"
              size="sm"
              onClick={toggleLightMode}
              className={theme.controls}
            >
              {isLightMode ? (
                <>
                  <Moon className="w-4 h-4 mr-2" />
                  Dark
                </>
              ) : (
                <>
                  <Sun className="w-4 h-4 mr-2" />
                  Light
                </>
              )}
            </Button>
          </div>

          <div className="flex items-center space-x-2">
            {/* Character Count */}
            <span className={`text-xs ${isLightMode ? 'text-gray-500' : 'text-[#666666]'}`}>
              {charCount} characters
            </span>

            {/* Distraction-Free Toggle */}
            <Button
              variant="ghost"
              size="sm"
              onClick={toggleDistractionFree}
              className={theme.controls}
            >
              {isDistractionFree ? (
                <>
                  <Minimize2 className="w-4 h-4 mr-2" />
                  Exit Focus
                </>
              ) : (
                <>
                  <Maximize2 className="w-4 h-4 mr-2" />
                  Focus Mode
                </>
              )}
            </Button>
          </div>
        </div>

        <div className={`relative textarea-container group ${isDistractionFree ? 'flex-1 flex flex-col' : ''}`}>
          <Textarea
            ref={textareaRef}
            id="journalEntry"
            value={journalContent}
            onChange={(e) => handleTextareaInput(e.target.value)}
            className={textareaClass}
            placeholder="Start writing here. Let it all out. No one will judge you. No one will remember..."
            style={{
              opacity: isBurning ? 0.3 : 1,
              transition: "opacity 0.3s ease",
              ...(isDistractionFree && { height: 'calc(100vh - 180px)', minHeight: '400px' })
            }}
          />
          
          {/* Gradient line effect on focus */}
          <div className={`absolute left-0 right-0 bottom-0 h-[2px] ${theme.gradient} transform scale-x-0 group-focus-within:scale-x-100 transition-transform duration-300 origin-center`}></div>
        </div>
        
        {/* Bottom Controls */}
        <div className={`flex justify-between items-center ${isDistractionFree ? 'mt-6' : 'mt-4'}`}>
          <div className="flex items-center space-x-4">
            {/* Timer component - use our local state */}
            {shouldShowTimer && (
              <Timer timeRemaining={displayTimeRemaining} />
            )}
          </div>
          
          <BurnButton 
            onClick={handleBurn} 
            disabled={journalContent.trim().length === 0 || isBurning}
          />
        </div>
      </div>

      {/* Only render the animation component when isAnimating is true, with a key to force remounting */}
      {isAnimating && (
        <Suspense fallback={<div className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center">
          <div className="text-orange-500 text-lg">Preparing animation...</div>
        </div>}>
          <FireAnimation 
            key={animationKey}
            text={journalBeforeBurn}
            style={animationStyle}
            onComplete={handleBurnComplete}
          />
        </Suspense>
      )}

      {showQuote && currentQuote && (
        <Suspense fallback={<div className="fixed inset-0 bg-black bg-opacity-75 flex items-center justify-center">
          <div className="bg-gray-800 p-4 rounded-lg animate-pulse">Loading quote...</div>
        </div>}>
          <MotivationalQuote 
            quote={currentQuote} 
            onClose={closeQuote} 
          />
        </Suspense>
      )}
    </>
  );
}
